# -*- coding: utf-8 -*-
#  Copyright (C) 2006, 2007, 2008, 2010 Stewart Adam
#  This file is part of fwfstab.

#  fwfstab is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  fwfstab is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with fwfstab; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
This file interfaces with the fstab.
"""
import os
import re
import shutil
#--
import fwfstab

class fstabLine:
  """Parses an entry in a fstab-formatted file"""
  # Initialize some variables to their default values
  __fs_spec = None
  __fs_file = None
  __fs_vfstype = None
  __fs_mntops = None
  __fs_freq = 0
  __fs_passno = 0
  __end_comment = ''
  
  def __init__(self, line):
    """Parses the fstab entry line constructs the object"""
    self.__rawtext = line.rstrip()
    # Parse the fstab line. The two digits groups are optional.
    match = re.match(r"^([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s*([0-9]+)?\s*([0-9]+)?\s*(#.*)?$", line.strip())
    if match:
      self.set_all_fields(*match.groups())
  
  def is_comment_or_whitespace(self):
    """Returns True if the fstab entry is comment"""
    entry_text = self.get_raw_entry_text()
    if not entry_text or entry_text.lstrip().startswith("#"):
      return True
    return False
      
    return self.get_raw_entry_text().lstrip().startswith("#")
  
  def is_parsable(self):
    """Returns True if the fstab entry is parsable (comments count as parsable)"""
    if not self.is_comment_or_whitespace() \
       and None in [self.__fs_spec, self.__fs_file, self.__fs_vfstype, \
                    self.__fs_mntops, self.__fs_freq, self.__fs_passno]:
      return False
    return True
  
  def get_all_fields(self):
    """Returns the fields for the fstab entry if the line was parsable.
    Otherwise, returns None."""
    if not self.is_parsable():
      raise ValueError("Entry is not parsable")
    return (self.__fs_spec, self.__fs_file, self.__fs_vfstype, self.__fs_mntops, self.__fs_freq, self.__fs_passno, self.__end_comment)
  
  def set_all_fields(self, fs_spec, fs_file, fs_vfstype, fs_mntops, fs_freq=0, fs_passno=0, end_comments=''):
    """Sets all fields in the fstab line"""
    self.__fs_spec, self.__fs_file, self.__fs_vfstype, self.__fs_mntops, \
    self.__fs_freq, self.__fs_passno, self.__end_comment = fs_spec, fs_file, \
      fs_vfstype, fs_mntops, fs_freq or 0, fs_passno or 0, end_comments or ''
  
  def get_raw_entry_text(self):
    """Return the raw text for the entry"""
    return self.__rawtext
  
  def generate_entry_text(self):
    """Return the generated text for the entry. If the entry was not parsable,
    a ValueError is raised."""
    if not self.is_parsable() and not self.is_comment_or_whitespace():
      raise ValueError("Entry is not parsable - cannot generate entry text")
    else:
      return ' '.join([self.__fs_spec, self.__fs_file, self.__fs_vfstype, self.__fs_mntops, self.__fs_freq, self.__fs_passno, self.__end_comment]).rstrip()

def read(fstabFile):
  """Read in fstab entires from a fstab-formatted file"""
  fh = open(fstabFile, 'r')
  lines = [fstabLine(line) for line in fh.readlines()]
  fh.close()
  return lines

def write(fstabFile, fstabEntries=[]):
  """Write a fstab-formatted file from the list of fstabLines. Return values of
  -1, 0 and 1 indicate a failure to create the backup fstab, failure writing to
  the fstab and a success respectively."""
  # create a backup copy of the fstab, returning False if we cannot
  backupFile = "%s.bak" % fstabFile
  if fwfstab.CheckPerms(backupFile):
    try:
      shutil.copyfile(fstabFile, backupFile)
    except IOError, OSError:
      return -1
  else:
    return -1
  # now we can safely write the new fstab file
  try:
    fh = open(fstabFile, 'w')
    for fstabEntry in fstabEntries:
      # if the line is parsable, then use our generated text
      # otherwise, stick the original line (either an invalid entry or a
      # comment) back into the fstab as-is.
      if fstabEntry.is_parsable() and not fstabEntry.is_comment_or_whitespace():
        fh.write(fstabEntry.generate_entry_text()+'\n')
      else:
        fh.write(fstabEntry.get_raw_entry_text()+'\n')
    fh.close()
  except IOError: # can't open fstab file
    return 0
  except:
    # should there be an error, restore the backup file and then complain
    shutil.copyfile(backupFile, fstabFile)
    raise
  return 1
